@@ -28,18 +28,19 @@ pub fn zero() -> Term { abs(abs(Var(1))) }
 ///
 /// # Example
 /// ```
+/// # #[macro_use] extern crate lambda_calculus;
+/// # fn main() {
 /// use lambda_calculus::arithmetic::is_zero;
 /// use lambda_calculus::booleans::{tru, fls};
 /// use lambda_calculus::reduction::beta_full;
 ///
-/// assert_eq!(beta_full(is_zero().app(0.into())), tru());
-/// assert_eq!(beta_full(is_zero().app(1.into())), fls());
+/// assert_eq!(beta_full(app!(is_zero(), 0.into())), tru());
+/// assert_eq!(beta_full(app!(is_zero(), 1.into())), fls());
+/// # }
 /// ```
 pub fn is_zero() -> Term {
     abs(
-        Var(1)
-        .app(abs(fls()))
-        .app(tru())
+        app!(Var(1), abs(fls()), tru())
     )
 }
 
@@ -56,7 +57,7 @@ pub fn is_zero() -> Term {
 /// ```
 pub fn one() -> Term {
     abs(abs(
-        Var(2).app(Var(1))
+        app!(Var(2), Var(1))
     ))
 }
 
@@ -75,11 +76,7 @@ pub fn one() -> Term {
 /// ```
 pub fn succ() -> Term {
     abs(abs(abs(
-        Var(2).app(
-            Var(3)
-            .app(Var(2))
-            .app(Var(1))
-        )
+        app!(Var(2), app!(Var(3), Var(2), Var(1)))
     )))
 }
 
@@ -89,18 +86,19 @@ pub fn succ() -> Term {
 ///
 /// # Example
 /// ```
+/// # #[macro_use] extern crate lambda_calculus;
+/// # fn main() {
 /// use lambda_calculus::arithmetic::plus;
 ///
-/// let mut expr = plus().app(3.into()).app(2.into());
+/// let mut expr = app!(plus(), 3.into(), 2.into());
 /// expr.beta_full();
 ///
 /// assert_eq!(expr, 5.into());
+/// # }
 /// ```
 pub fn plus() -> Term {
     abs(abs(abs(abs(
-        Var(4)
-        .app(Var(2))
-        .app(Var(3).app(Var(2)).app(Var(1)))
+        app!(Var(4), Var(2), app!(Var(3), Var(2), Var(1)))
     ))))
 }
 
@@ -110,16 +108,19 @@ pub fn plus() -> Term {
 ///
 /// # Example
 /// ```
+/// # #[macro_use] extern crate lambda_calculus;
+/// # fn main() {
 /// use lambda_calculus::arithmetic::mult;
 ///
-/// let mut expr = mult().app(2.into()).app(3.into());
+/// let mut expr = app!(mult(), 2.into(), 3.into());
 /// expr.beta_full();
 ///
 /// assert_eq!(expr, 6.into());
+/// # }
 /// ```
 pub fn mult() -> Term {
     abs(abs(abs(
-        Var(3).app(Var(2).app(Var(1)))
+        app!(Var(3), app!(Var(2), Var(1)))
     )))
 }
 
@@ -129,19 +130,19 @@ pub fn mult() -> Term {
 ///
 /// # Example
 /// ```
+/// # #[macro_use] extern crate lambda_calculus;
+/// # fn main() {
 /// use lambda_calculus::arithmetic::pow;
 ///
-/// let mut expr = pow().app(2.into()).app(3.into());
+/// let mut expr = app!(pow(), 2.into(), 3.into());
 /// expr.beta_full();
 ///
 /// assert_eq!(expr, 8.into());
+/// # }
 /// ```
 pub fn pow() -> Term {
     abs(abs(
-        is_zero()
-        .app(Var(1))
-        .app(one())
-        .app(Var(1).app(Var(2)))
+        app!(is_zero(), Var(1), one(), app!(Var(1), Var(2)))
     ))
 }
 
@@ -160,10 +161,12 @@ pub fn pow() -> Term {
 /// ```
 pub fn pred() -> Term {
     abs(abs(abs(
-        Var(3)
-        .app(abs(abs(Var(1).app(Var(2).app(Var(4))))))
-        .app(abs(Var(2)))
-        .app(abs(Var(1)))
+        app!(
+        Var(3),
+        abs(abs(app!(Var(1), app!(Var(2), Var(4))))),
+        abs(Var(2)),
+        abs(Var(1))
+        )
     )))
 }
 
@@ -173,16 +176,19 @@ pub fn pred() -> Term {
 ///
 /// # Example
 /// ```
+/// # #[macro_use] extern crate lambda_calculus;
+/// # fn main() {
 /// use lambda_calculus::arithmetic::sub;
 ///
-/// let mut expr = sub().app(5.into()).app(3.into());
+/// let mut expr = app!(sub(), 5.into(), 3.into());
 /// expr.beta_full();
 ///
 /// assert_eq!(expr, 2.into());
+/// # }
 /// ```
 pub fn sub() -> Term {
     abs(abs(
-        Var(1).app(pred()).app(Var(2))
+        app!(Var(1), pred(), Var(2))
     ))
 }
 
@@ -193,18 +199,21 @@ pub fn sub() -> Term {
 ///
 /// # Examples
 /// ```
+/// # #[macro_use] extern crate lambda_calculus;
+/// # fn main() {
 /// use lambda_calculus::arithmetic::lt;
 /// use lambda_calculus::booleans::{tru, fls};
 /// use lambda_calculus::reduction::beta_full;
 ///
-/// assert_eq!(beta_full(lt().app(0.into()).app(0.into())), fls());
-/// assert_eq!(beta_full(lt().app(1.into()).app(1.into())), fls());
-/// assert_eq!(beta_full(lt().app(0.into()).app(1.into())), tru());
-/// assert_eq!(beta_full(lt().app(1.into()).app(0.into())), fls());
+/// assert_eq!(beta_full(app!(lt(), 0.into(), 0.into())), fls());
+/// assert_eq!(beta_full(app!(lt(), 1.into(), 1.into())), fls());
+/// assert_eq!(beta_full(app!(lt(), 0.into(), 1.into())), tru());
+/// assert_eq!(beta_full(app!(lt(), 1.into(), 0.into())), fls());
+/// # }
 /// ```
 pub fn lt() -> Term {
     abs(abs(
-        not().app(leq().app(Var(1)).app(Var(2)))
+        app!(not(), app!(leq(), Var(1), Var(2)))
     ))
 }
 
@@ -215,18 +224,21 @@ pub fn lt() -> Term {
 ///
 /// # Examples
 /// ```
+/// # #[macro_use] extern crate lambda_calculus;
+/// # fn main() {
 /// use lambda_calculus::arithmetic::leq;
 /// use lambda_calculus::booleans::{tru, fls};
 /// use lambda_calculus::reduction::beta_full;
 ///
-/// assert_eq!(beta_full(leq().app(0.into()).app(0.into())), tru());
-/// assert_eq!(beta_full(leq().app(1.into()).app(1.into())), tru());
-/// assert_eq!(beta_full(leq().app(0.into()).app(1.into())), tru());
-/// assert_eq!(beta_full(leq().app(1.into()).app(0.into())), fls());
+/// assert_eq!(beta_full(app!(leq(), 0.into(), 0.into())), tru());
+/// assert_eq!(beta_full(app!(leq(), 1.into(), 1.into())), tru());
+/// assert_eq!(beta_full(app!(leq(), 0.into(), 1.into())), tru());
+/// assert_eq!(beta_full(app!(leq(), 1.into(), 0.into())), fls());
+/// # }
 /// ```
 pub fn leq() -> Term {
     abs(abs(
-        is_zero().app(sub().app(Var(2)).app(Var(1)))
+        app!(is_zero(), app!(sub(), Var(2), Var(1)))
     ))
 }
 
@@ -237,20 +249,25 @@ pub fn leq() -> Term {
 ///
 /// # Examples
 /// ```
+/// # #[macro_use] extern crate lambda_calculus;
+/// # fn main() {
 /// use lambda_calculus::arithmetic::eq;
 /// use lambda_calculus::booleans::{tru, fls};
 /// use lambda_calculus::reduction::beta_full;
 ///
-/// assert_eq!(beta_full(eq().app(0.into()).app(0.into())), tru());
-/// assert_eq!(beta_full(eq().app(1.into()).app(1.into())), tru());
-/// assert_eq!(beta_full(eq().app(0.into()).app(1.into())), fls());
-/// assert_eq!(beta_full(eq().app(1.into()).app(0.into())), fls());
+/// assert_eq!(beta_full(app!(eq(), 0.into(), 0.into())), tru());
+/// assert_eq!(beta_full(app!(eq(), 1.into(), 1.into())), tru());
+/// assert_eq!(beta_full(app!(eq(), 0.into(), 1.into())), fls());
+/// assert_eq!(beta_full(app!(eq(), 1.into(), 0.into())), fls());
+/// # }
 /// ```
 pub fn eq() -> Term {
     abs(abs(
-        and()
-        .app(leq().app(Var(2)).app(Var(1)))
-        .app(leq().app(Var(1)).app(Var(2)))
+        app!(
+            and(),
+            app!(leq(), Var(2), Var(1)),
+            app!(leq(), Var(1), Var(2))
+        )
     ))
 }
 
@@ -261,20 +278,25 @@ pub fn eq() -> Term {
 ///
 /// # Examples
 /// ```
+/// # #[macro_use] extern crate lambda_calculus;
+/// # fn main() {
 /// use lambda_calculus::arithmetic::neq;
 /// use lambda_calculus::booleans::{tru, fls};
 /// use lambda_calculus::reduction::beta_full;
 ///
-/// assert_eq!(beta_full(neq().app(0.into()).app(0.into())), fls());
-/// assert_eq!(beta_full(neq().app(1.into()).app(1.into())), fls());
-/// assert_eq!(beta_full(neq().app(0.into()).app(1.into())), tru());
-/// assert_eq!(beta_full(neq().app(1.into()).app(0.into())), tru());
+/// assert_eq!(beta_full(app!(neq(), 0.into(), 0.into())), fls());
+/// assert_eq!(beta_full(app!(neq(), 1.into(), 1.into())), fls());
+/// assert_eq!(beta_full(app!(neq(), 0.into(), 1.into())), tru());
+/// assert_eq!(beta_full(app!(neq(), 1.into(), 0.into())), tru());
+/// # }
 /// ```
 pub fn neq() -> Term {
     abs(abs(
-        or()
-        .app(not().app(leq().app(Var(2)).app(Var(1))))
-        .app(not().app(leq().app(Var(1)).app(Var(2))))
+        app!(
+            or(),
+            app!(not(), app!(leq(), Var(2), Var(1))),
+            app!(not(), app!(leq(), Var(1), Var(2)))
+        )
     ))
 }
 
@@ -285,18 +307,21 @@ pub fn neq() -> Term {
 ///
 /// # Examples
 /// ```
+/// # #[macro_use] extern crate lambda_calculus;
+/// # fn main() {
 /// use lambda_calculus::arithmetic::geq;
 /// use lambda_calculus::booleans::{tru, fls};
 /// use lambda_calculus::reduction::beta_full;
 ///
-/// assert_eq!(beta_full(geq().app(0.into()).app(0.into())), tru());
-/// assert_eq!(beta_full(geq().app(1.into()).app(1.into())), tru());
-/// assert_eq!(beta_full(geq().app(0.into()).app(1.into())), fls());
-/// assert_eq!(beta_full(geq().app(1.into()).app(0.into())), tru());
+/// assert_eq!(beta_full(app!(geq(), 0.into(), 0.into())), tru());
+/// assert_eq!(beta_full(app!(geq(), 1.into(), 1.into())), tru());
+/// assert_eq!(beta_full(app!(geq(), 0.into(), 1.into())), fls());
+/// assert_eq!(beta_full(app!(geq(), 1.into(), 0.into())), tru());
+/// # }
 /// ```
 pub fn geq() -> Term {
     abs(abs(
-        leq().app(Var(1)).app(Var(2))
+        app!(leq(), Var(1), Var(2))
     ))
 }
 
@@ -307,18 +332,21 @@ pub fn geq() -> Term {
 ///
 /// # Examples
 /// ```
+/// # #[macro_use] extern crate lambda_calculus;
+/// # fn main() {
 /// use lambda_calculus::arithmetic::{zero, one, gt};
 /// use lambda_calculus::booleans::{tru, fls};
 /// use lambda_calculus::reduction::beta_full;
 ///
-/// assert_eq!(beta_full(gt().app(0.into()).app(0.into())), fls());
-/// assert_eq!(beta_full(gt().app(1.into()).app(1.into())), fls());
-/// assert_eq!(beta_full(gt().app(0.into()).app(1.into())), fls());
-/// assert_eq!(beta_full(gt().app(1.into()).app(0.into())), tru());
+/// assert_eq!(beta_full(app!(gt(), 0.into(), 0.into())), fls());
+/// assert_eq!(beta_full(app!(gt(), 1.into(), 1.into())), fls());
+/// assert_eq!(beta_full(app!(gt(), 0.into(), 1.into())), fls());
+/// assert_eq!(beta_full(app!(gt(), 1.into(), 0.into())), tru());
+/// # }
 /// ```
 pub fn gt() -> Term {
     abs(abs(
-        not().app(leq().app(Var(2)).app(Var(1)))
+        app!(not(), app!(leq(), Var(2), Var(1)))
     ))
 }
 
@@ -330,31 +358,36 @@ pub fn gt() -> Term {
 ///
 /// # Example
 /// ```
+/// # #[macro_use] extern crate lambda_calculus;
+/// # fn main() {
 /// use lambda_calculus::arithmetic::div;
 /// use lambda_calculus::term::Term;
 ///
-/// let mut expr = div().app(5.into()).app(2.into());
+/// let mut expr = app!(div(), 5.into(), 2.into());
 /// expr.beta_full();
 ///
 /// assert_eq!(expr, Term::from((2.into(), 1.into())));
+/// # }
 /// ```
 pub fn div() -> Term {
-    y()
-    .app(
+    app!(
+        y(),
         abs(abs(abs(abs(
-            lt()
-            .app(Var(2))
-            .app(Var(1))
-            .app(pair().app(Var(3)).app(Var(2)))
-            .app(
-                Var(4)
-                .app(succ().app(Var(3)))
-                .app(sub().app(Var(2)).app(Var(1)))
-                .app(Var(1))
+            app!(
+                lt(),
+                Var(2),
+                Var(1),
+                app!(pair(), Var(3), Var(2)),
+                app!(
+                    Var(4),
+                    app!(succ(), Var(3)),
+                    app!(sub(), Var(2), Var(1)),
+                    Var(1)
+                )
             )
-        ))))
+        )))),
+        zero()
     )
-    .app(zero())
 }
 
 /// Applied to two Church-encoded numbers it returns a Church-encoded quotient of their division.
@@ -365,29 +398,32 @@ pub fn div() -> Term {
 ///
 /// # Example
 /// ```
+/// # #[macro_use] extern crate lambda_calculus;
+/// # fn main() {
 /// use lambda_calculus::arithmetic::quot;
 ///
-/// let mut expr = quot().app(6.into()).app(2.into());
+/// let mut expr = app!(quot(), 6.into(), 2.into());
 /// expr.beta_full();
 ///
 /// assert_eq!(expr, 3.into());
+/// # }
 /// ```
 pub fn quot() -> Term {
-    y().app(
+    app!(
+        y(),
         abs(abs(abs(
-            lt()
-            .app(Var(2))
-            .app(Var(1))
-            .app(zero())
-            .app(
-                succ().app(
-                    Var(3)
-                    .app(
-                        sub()
-                        .app(Var(2))
-                        .app(Var(1))
+            app!(
+                lt(),
+                Var(2),
+                Var(1),
+                zero(),
+                app!(
+                    succ(),
+                    app!(
+                        Var(3),
+                        app!(sub(), Var(2), Var(1)),
+                        Var(1)
                     )
-                    .app(Var(1))
                 )
             )
         )))
@@ -402,20 +438,19 @@ pub fn quot() -> Term {
 ///
 /// # Example
 /// ```
+/// # #[macro_use] extern crate lambda_calculus;
+/// # fn main() {
 /// use lambda_calculus::arithmetic::rem;
 ///
-/// let mut expr = rem().app(3.into()).app(2.into());
+/// let mut expr = app!(rem(), 3.into(), 2.into());
 /// expr.beta_full();
 ///
 /// assert_eq!(expr, 1.into());
+/// # }
 /// ```
 pub fn rem() -> Term {
     abs(abs(
-        second().app(
-            div()
-            .app(Var(2))
-            .app(Var(1))
-        )
+        app!(second(), app!(div(), Var(2), Var(1)))
     ))
 }
 
@@ -434,17 +469,20 @@ pub fn rem() -> Term {
 /// assert_eq!(expr, 6.into());
 /// ```
 pub fn factorial() -> Term {
-    y()
-    .app(
+    app!(
+        y(),
         abs(abs(
-            is_zero()
-            .app(Var(1))
-            .app(one())
-            .app(
-                mult()
-                .app(Var(1))
-                .app(
-                    Var(2).app(pred().app(Var(1)))
+            app!(
+                is_zero(),
+                Var(1),
+                one(),
+                app!(
+                    mult(),
+                    Var(1),
+                    app!(
+                        Var(2),
+                        app!(pred(), Var(1))
+                    )
                 )
             )
         ))
@@ -469,14 +507,14 @@ impl Term {
     }
 
     fn _value(&self) -> Result<usize, Error> {
-        if let Ok(ref rhs) = self.rhs_ref() {
-            Ok(1 + try!(rhs._value()))
-        } else if let Var(n) = *self {
-            if n == 1 {
-                Ok(0)
+        if let Ok((lhs, rhs)) = self.unapp_ref() {
+            if *lhs == Var(2) {
+                Ok(1 + try!(rhs._value()))
             } else {
                 Err(NotANum)
             }
+        } else if *self == Var(1) {
+            Ok(0)
         } else {
             Err(NotANum)
         }
@@ -515,8 +553,8 @@ mod test {
 
     #[test]
     fn church_invalid_nums() {
-        assert_eq!(tru().is_cnum(),       false);
-        assert_eq!(Var(1).is_cnum(),      false);
+        assert_eq!(      tru().is_cnum(), false);
+        assert_eq!(     Var(1).is_cnum(), false);
         assert_eq!(abs(Var(1)).is_cnum(), false);
     }
 
@@ -527,74 +565,74 @@ mod test {
 
     #[test]
     fn church_successor() {
-        assert_eq!(beta_full(succ().app(0.into())), 1.into());
-        assert_eq!(beta_full(succ().app(1.into())), 2.into());
-        assert_eq!(beta_full(succ().app(2.into())), 3.into());
+        assert_eq!(beta_full(app!(succ(), 0.into())), 1.into());
+        assert_eq!(beta_full(app!(succ(), 1.into())), 2.into());
+        assert_eq!(beta_full(app!(succ(), 2.into())), 3.into());
     }
 
     #[test]
     fn church_predecessor() {
-        assert_eq!(beta_full(pred().app(0.into())), 0.into());
-        assert_eq!(beta_full(pred().app(1.into())), 0.into());
-        assert_eq!(beta_full(pred().app(5.into())), 4.into());
+        assert_eq!(beta_full(app!(pred(), 0.into())), 0.into());
+        assert_eq!(beta_full(app!(pred(), 1.into())), 0.into());
+        assert_eq!(beta_full(app!(pred(), 5.into())), 4.into());
     }
 
     #[test]
     fn church_plus_sub_equivalents() {
-        assert_eq!(beta_full(plus().app(1.into())), succ()); // PLUS 1 → SUCC
-        assert_eq!(beta_full(c().app(sub()).app(1.into())), pred()); // C SUB 1 → PRED
+        assert_eq!(beta_full(app!(    plus(), 1.into())), succ()); // PLUS 1 → SUCC
+        assert_eq!(beta_full(app!(c(), sub(), 1.into())), pred()); // C SUB 1 → PRED
     }
 
     #[test]
     fn church_multiplication() {
-        assert_eq!(beta_full(mult().app(3.into()).app(4.into())), 12.into());
-        assert_eq!(beta_full(mult().app(1.into()).app(3.into())), 3.into());
-        assert_eq!(beta_full(mult().app(3.into()).app(1.into())), 3.into());
-        assert_eq!(beta_full(mult().app(5.into()).app(0.into())), 0.into());
-        assert_eq!(beta_full(mult().app(0.into()).app(5.into())), 0.into());
+        assert_eq!(beta_full(app!(mult(), 3.into(), 4.into())), 12.into());
+        assert_eq!(beta_full(app!(mult(), 1.into(), 3.into())), 3.into());
+        assert_eq!(beta_full(app!(mult(), 3.into(), 1.into())), 3.into());
+        assert_eq!(beta_full(app!(mult(), 5.into(), 0.into())), 0.into());
+        assert_eq!(beta_full(app!(mult(), 0.into(), 5.into())), 0.into());
     }
 
     #[test]
     fn church_exponentiation() {
-        assert_eq!(beta_full(pow().app(2.into()).app(4.into())), 16.into());
-        assert_eq!(beta_full(pow().app(1.into()).app(3.into())), 1.into());
-        assert_eq!(beta_full(pow().app(3.into()).app(1.into())), 3.into());
-        assert_eq!(beta_full(pow().app(5.into()).app(0.into())), 1.into());
-        assert_eq!(beta_full(pow().app(0.into()).app(5.into())), 0.into());
+        assert_eq!(beta_full(app!(pow(), 2.into(), 4.into())), 16.into());
+        assert_eq!(beta_full(app!(pow(), 1.into(), 3.into())), 1.into());
+        assert_eq!(beta_full(app!(pow(), 3.into(), 1.into())), 3.into());
+        assert_eq!(beta_full(app!(pow(), 5.into(), 0.into())), 1.into());
+        assert_eq!(beta_full(app!(pow(), 0.into(), 5.into())), 0.into());
     }
 
     #[test]
     fn church_division() {
-        assert_eq!(beta_full(div().app(2.into()).app(2.into())), (1.into(), 0.into()).into());
-        assert_eq!(beta_full(div().app(3.into()).app(2.into())), (1.into(), 1.into()).into());
-        assert_eq!(beta_full(div().app(2.into()).app(1.into())), (2.into(), 0.into()).into());
-        assert_eq!(beta_full(div().app(0.into()).app(3.into())), (0.into(), 0.into()).into());
-        // assert_eq!(beta_full(div().app(1.into()).app(0.into())), ); division by 0 hangs
+        assert_eq!(beta_full(app!(div(), 2.into(), 2.into())), (1.into(), 0.into()).into());
+        assert_eq!(beta_full(app!(div(), 3.into(), 2.into())), (1.into(), 1.into()).into());
+        assert_eq!(beta_full(app!(div(), 2.into(), 1.into())), (2.into(), 0.into()).into());
+        assert_eq!(beta_full(app!(div(), 0.into(), 3.into())), (0.into(), 0.into()).into());
+        // assert_eq!(beta_full(app!(div(), 1.into(), 0.into())), ); division by 0 hangs
     }
 
     #[test]
     fn church_quotient() {
-        assert_eq!(beta_full(quot().app(2.into()).app(2.into())), 1.into());
-        assert_eq!(beta_full(quot().app(3.into()).app(2.into())), 1.into());
-        assert_eq!(beta_full(quot().app(2.into()).app(1.into())), 2.into());
-        assert_eq!(beta_full(quot().app(0.into()).app(3.into())), 0.into());
-        // assert_eq!(beta_full(quot().app(1.into()).app(0.into())), ); division by 0 hangs
+        assert_eq!(beta_full(app!(quot(), 2.into(), 2.into())), 1.into());
+        assert_eq!(beta_full(app!(quot(), 3.into(), 2.into())), 1.into());
+        assert_eq!(beta_full(app!(quot(), 2.into(), 1.into())), 2.into());
+        assert_eq!(beta_full(app!(quot(), 0.into(), 3.into())), 0.into());
+        // assert_eq!(beta_full(app!(quot(), 1.into(), 0.into())), ); division by 0 hangs
     }
 
     #[test]
     fn church_remainder() {
-        assert_eq!(beta_full(rem().app(2.into()).app(2.into())), 0.into());
-        assert_eq!(beta_full(rem().app(3.into()).app(2.into())), 1.into());
-        assert_eq!(beta_full(rem().app(2.into()).app(1.into())), 0.into());
-        assert_eq!(beta_full(rem().app(0.into()).app(3.into())), 0.into());
-        // assert_eq!(beta_full(quot().app(1.into()).app(0.into())), ); division by 0 hangs
+        assert_eq!(beta_full(app!(rem(), 2.into(), 2.into())), 0.into());
+        assert_eq!(beta_full(app!(rem(), 3.into(), 2.into())), 1.into());
+        assert_eq!(beta_full(app!(rem(), 2.into(), 1.into())), 0.into());
+        assert_eq!(beta_full(app!(rem(), 0.into(), 3.into())), 0.into());
+        // assert_eq!(beta_full(app!(rem(), 1.into(), 0.into())), ); division by 0 hangs
     }
 
     #[test]
     fn church_factorial() {
-        assert_eq!(beta_full(factorial().app(0.into())), 1.into());
-        assert_eq!(beta_full(factorial().app(1.into())), 1.into());
-        assert_eq!(beta_full(factorial().app(2.into())), 2.into());
-        assert_eq!(beta_full(factorial().app(3.into())), 6.into());
+        assert_eq!(beta_full(app!(factorial(), 0.into())), 1.into());
+        assert_eq!(beta_full(app!(factorial(), 1.into())), 1.into());
+        assert_eq!(beta_full(app!(factorial(), 2.into())), 2.into());
+        assert_eq!(beta_full(app!(factorial(), 3.into())), 6.into());
     }
 }
